/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef CLIENTD_HPP
#define CLIENTD_HPP

#include <string>
#include <vector>
#include <stdint.h>
#include <tr1/memory>

#include "clientd_queue.hpp"
#include "errors.hpp"

// TODO move these to a different file when a server is created
enum response_status { STATUS_SUCCESS, STATUS_KEY_UNDEFINED,
    STATUS_ERROR, STATUS_CLIENTD_ERROR, STATUS_CONN_ERROR,
    STATUS_UNAVAILABLE };

/** Make a request to the client daemon, return result.
 *
 * If this is a 'get' request and the size returned is nonzero, the pointer
 * returned was allocated and should be deleted with delete[].  Otherwise,
 * both values in the result will be zero.
 */
std::pair<uint8_t *, size_t>
	make_request_clientd(
	    const std::vector<std::pair<std::string, uint16_t> > &hosts,
	    const std::string &key, const uint8_t *value, size_t value_size)
	    throw (bad_message, client_error, key_undefined, server_error,
	    unix_error);

// start listening, return socket
int	start_listening() throw (unix_error);

// argument for clientd_listener
struct clientd_listener_args {
	clientd_listener_args(Clientd_queue &queue, int socket) :
		queue(queue),
		socket(socket)
	{}

	~clientd_listener_args()
	{
		if (socket >= 0) close(socket);
	}

	Clientd_queue	&queue;
	int		socket;
};
// let listener run
void	*clientd_listener(void *voidargs) throw (client_error, unix_error);

void	send_response(const uint8_t *value, size_t value_size,
	    enum response_status status, uint16_t http_status,
	    const Clientd_connection &cx)
	    throw (unix_error);

#endif
