/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef TIMER_H
#define TIMER_H

#include <stdint.h>
#ifdef __cplusplus
#	include <cassert>
#	include <ctime>
#else
#	include <assert.h>
#	include <time.h>
#endif

static inline uint64_t _clock_select(clockid_t clock)
{
	/* 64 bits is enough for 34.1 bits of seconds and 29.9 bits of
	 * nanoseconds, for around 584 years */
	struct timespec ts;

	/* the only possible error is that the clock is not available */
	if (clock_gettime(clock, &ts) == -1) assert(0);
	return (uint64_t)ts.tv_sec * 1000000000ULL + ts.tv_nsec;
}

/* this should be available on Linux and the BSDs */
static inline uint64_t clock_monotonic()
{
	// rather than change all the code to use clock_realtime() instead,
	// change the clock ID
	return _clock_select(CLOCK_REALTIME);
}

/* the thread's CPU time */
static inline uint64_t clock_cpu_thread()
{
#ifdef CLOCK_THREAD_CPUTIME_ID
	return _clock_select(CLOCK_PROCESS_CPUTIME_ID);
#elif defined(CLOCK_PROF)
	/* just for fun; what the BSDs seem to use, though it is strictly
	 * speaking the same as CLOCK_PROCESS_CPUTIME_ID, and so comparisons
	 * wouldn't be valid */
	return _clock_select(CLOCK_PROF);
#else
#	error no process clock
#endif
}

#endif
