/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <netdb.h>

#include <cerrno>
#include <cstring>
#include <sstream>

#include "errors.hpp"

bad_message::bad_message(const std::string &msg) : _msg("bad message: ")
{
	_msg += msg;
}

client_error::client_error(const std::string &msg) : _msg("client error")
{
	if (!msg.empty()) {
		_msg += ": ";
		_msg += msg;
	}
}

http_parse_error::http_parse_error(const std::string &msg, unsigned line) :
	_msg("HTTP parse error")
{
	std::ostringstream out;
	if (line != static_cast<unsigned>(-1))
		out << " on line " << line;
	out << ": " << msg;
	_msg += out.str();
}

lookup_error::lookup_error(const std::pair<std::string, uint16_t> &host,
    int errnum) :
	_msg("lookup failed for ")
{
	std::ostringstream out;
	out << host.first << ':' << host.second << ": " << hstrerror(errnum);
	_msg += out.str();
}

parse_error::parse_error(const std::string &msg, unsigned line) :
	_msg("parse error")
{
	std::ostringstream out;
	if (line)
		out << " on line " << line;
	out << ": " << msg;
	_msg += out.str();
}

request_restart::request_restart(const std::string &msg) :
		_msg("request restart")
{

}

server_error::server_error(const std::string &msg) : _msg("server error")
{
	if (!msg.empty()) {
		_msg += ": ";
		_msg += msg;
	}
}

unix_error::unix_error(const std::string &msg, int errnum) :
	_msg(msg),
	_errnum(errnum)
{
	_msg += ": ";
	_msg += strerror(_errnum);
}

unix_error::unix_error(const std::string &msg) :
	_msg(msg),
	_errnum(errno)
{
	_msg += ": ";
	_msg += strerror(_errnum);
}
