/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef MD5_HPP
#define MD5_HPP

#include <endian.h>

#include <stdint.h>
#include <string>

#include <openssl/md5.h>

// transforms the MD5 hash into two 64bit integers
inline void
md5_as_64bit(const uint8_t in[MD5_DIGEST_LENGTH], uint64_t out[2])
{
	uint8_t *out8 = reinterpret_cast<uint8_t *>(out);
	for (size_t i = 0; i < MD5_DIGEST_LENGTH; i++)
		out8[i] = in[i];
#ifdef NBO
	out[0] = be64toh(out[0]);
	out[1] = be64toh(out[1]);
#endif
}

class MD5_hash {
public:
	MD5_hash()
	{
		init();
	}

	// call before calculating a second hash
	void init()
	{
		MD5_Init(&ctx);
	}

	void update(const void *data, size_t sz)
	{
		MD5_Update(&ctx, data, sz);
	}
	void update(const std::string &str)
	{
		update(str.c_str(), str.size());
	}

	// only call a final() once, so make it count
	void final(uint8_t result[MD5_DIGEST_LENGTH])
	{
		MD5_Final(result, &ctx);
	}
	// return just the most significant 64 bits of the hash; return value
	// is in host-endian
	uint64_t final64()
	{
		uint64_t digest[2];
		MD5_Final(reinterpret_cast<uint8_t *>(digest), &ctx);
#ifdef NBO
		return be64toh(digest[0]);
#else
		return digest[0];
#endif
	}

private:
	MD5_CTX ctx;
};

// just compute the hash over a single buffer
inline void md5_hash(const void *data, size_t sz,
    uint8_t result[MD5_DIGEST_LENGTH])
{
	MD5_hash md5;
	md5.update(data, sz);
	md5.final(result);
}
inline void md5_hash(const std::string &str,
    uint8_t result[MD5_DIGEST_LENGTH])
{
	md5_hash(str.c_str(), str.size(), result);
}

#endif
