/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef PARSE_HPP
#define PARSE_HPP

#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>

#include <stdint.h>
#include <string>
#include <vector>

#include "errors.hpp"

const char *const CLIENTD_CONFIG = "servers.txt";
const char *const SERVER_CONFIG = "servers.txt";

void	resolve_hosts(
	    const std::vector<std::pair<std::string, uint16_t> > &hosts,
	    std::vector<struct sockaddr_in> &out_addrs)
	    throw (lookup_error);

// return a vector of (host, port) pairs
void	config_parse(const std::string &path,
	    std::vector<std::pair<std::string, uint16_t> > &out_hosts)
	    throw (parse_error, unix_error);

// parse VALUE from the HTTP property:
//	Pragma: servers=VALUE
// where VALUE is ADDR0:PORT0,ADDR1:PORT1,...
void	pragma_servers_parse(const std::string &value,
	    std::vector<std::pair<std::string, uint16_t> > &out_hosts)
	    throw (parse_error);

#endif
