/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>, Jeffery Roller <roller-j@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>

#include <cassert>
#include <cerrno>
#include <cstring>
#include <iostream>

#include "server_comm.hpp"

void
send_put_server(const std::string &message, int socket)
	throw (unix_error){

	const char * buf = message.c_str();
	size_t total = message.size();

	send_put_server_n(buf, total, socket);
}

void
send_put_server_n(const char * buf, size_t total, int socket)
		throw (unix_error){
	size_t count = 0;
	while (count < total) {
		ssize_t bytes;
		if ((bytes = write(socket, buf + count, total - count))
		    == -1)
			throw unix_error("write failed");
		count += bytes;
}	
		
		
		
}

int
send_to_addr(const std::string &message, const sockaddr_in * addr)
	throw (unix_error){

	int sock;
	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0){
		throw unix_error("socket failed");
	}
	if (connect(sock, (struct sockaddr *)addr,
	    sizeof(struct sockaddr_in)) == -1){
		long error = errno;
		close(sock);
#ifdef SERVER_DEBUG
		std::cerr << "Connect failed inside send_to_addr()\n";
#endif
		throw unix_error("connect failed", error);
	}
	try {
		send_put_server(message, sock);
	} catch (...) {
		close(sock);
		throw;
	}
	return sock;
}

int
send_to_addr_n(const char * message, unsigned int n, const sockaddr_in * addr)
		throw (unix_error){
	int sock;
	if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0){
		throw unix_error("socket failed");
}
	if (connect(sock, (struct sockaddr *)addr,
	    sizeof(struct sockaddr_in)) == -1){
		long error = errno;
		close(sock);
#ifdef SERVER_DEBUG
		std::cerr << "Connect failed inside send_to_addr()\n";
#endif
		throw unix_error("connect failed", error);
}
	try {
		send_put_server_n(message, n, sock);
} catch (...) {
		close(sock);
		throw;
}
	return sock;
		
		
}
